@@ -25,12 +25,6 @@ CONF = cfg.CONF
 
 class InfoController(object):
     def get_image_import(self, req):
-        # TODO(jokke): Will be removed after the config option
-        # is removed. (deprecated)
-        if not CONF.enable_image_import:
-            msg = _("Image import is not supported at this site.")
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
         # TODO(jokke): All the rest of the boundaries should be implemented.
         import_methods = {
             'description': 'Import methods available.',
@@ -42,6 +36,33 @@ class InfoController(object):
             'import-methods': import_methods
         }
 
+    def get_stores(self, req):
+        # TODO(abhishekk): This will be removed after config options
+        # 'stores' and 'default_store' are removed.
+        enabled_backends = CONF.enabled_backends
+        if not enabled_backends:
+            msg = _("Multi backend is not supported at this site.")
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        backends = []
+        for backend in enabled_backends:
+            if backend.startswith("os_glance_"):
+                continue
+
+            stores = {}
+            stores['id'] = backend
+            description = getattr(CONF, backend).store_description
+            if description:
+                stores['description'] = description
+            if backend == CONF.glance_store.default_backend:
+                stores['default'] = "true"
+            # Check if http store is configured then mark it as read-only
+            if enabled_backends[backend] == 'http':
+                stores['read-only'] = "true"
+            backends.append(stores)
+
+        return {'stores': backends}
+
 
 def create_resource():
     return wsgi.Resource(InfoController())
