@@ -15,20 +15,21 @@
 import glance_store
 from oslo_log import log as logging
 from oslo_utils import encodeutils
+from six.moves import http_client as http
 import webob.exc
 
 from glance.api import policy
+from glance.api.v2 import images as v2_api
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
-from glance import i18n
+from glance.i18n import _
 import glance.notifier
 
 
 LOG = logging.getLogger(__name__)
-_ = i18n._
 
 
 class Controller(object):
@@ -89,14 +90,26 @@ class Controller(object):
 
 class ResponseSerializer(wsgi.JSONResponseSerializer):
     def update(self, response, result):
-        response.status_int = 204
+        response.status_int = http.NO_CONTENT
 
     def delete(self, response, result):
-        response.status_int = 204
+        response.status_int = http.NO_CONTENT
+
+
+class RequestDeserializer(wsgi.JSONRequestDeserializer):
+    def update(self, request):
+        try:
+            schema = v2_api.get_schema()
+            schema_format = {"tags": [request.urlvars.get('tag_value')]}
+            schema.validate(schema_format)
+        except exception.InvalidObject as e:
+            raise webob.exc.HTTPBadRequest(explanation=e.msg)
+        return super(RequestDeserializer, self).default(request)
 
 
 def create_resource():
     """Images resource factory method"""
     serializer = ResponseSerializer()
+    deserializer = RequestDeserializer()
     controller = Controller()
-    return wsgi.Resource(controller, serializer=serializer)
+    return wsgi.Resource(controller, deserializer, serializer)
