@@ -17,10 +17,11 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from six.moves import http_client
+from six.moves import urllib
 import webob.dec
 
 from glance.common import wsgi
-from glance.i18n import _, _LW
+from glance.i18n import _
 
 
 versions_opts = [
@@ -59,22 +60,36 @@ class Controller(object):
     def index(self, req, explicit=False):
         """Respond to a request for all OpenStack API versions."""
         def build_version_object(version, path, status):
-            url = CONF.public_endpoint or req.host_url
+            url = CONF.public_endpoint or req.application_url
+            # Always add '/' to url end for urljoin href url
+            url = url.rstrip('/') + '/'
+            href = urllib.parse.urljoin(url, path).rstrip('/') + '/'
             return {
                 'id': 'v%s' % version,
                 'status': status,
                 'links': [
                     {
                         'rel': 'self',
-                        'href': '%s/%s/' % (url, path),
+                        'href': '%s' % href,
                     },
                 ],
             }
 
         version_objs = []
         if CONF.enable_v2_api:
+            if CONF.enabled_backends:
+                version_objs.extend([
+                    build_version_object(2.10, 'v2', 'CURRENT'),
+                    build_version_object(2.9, 'v2', 'SUPPORTED'),
+                    build_version_object(2.8, 'v2', 'SUPPORTED')
+                ])
+            else:
+                version_objs.extend([
+                    build_version_object(2.9, 'v2', 'CURRENT'),
+                ])
             version_objs.extend([
-                build_version_object(2.6, 'v2', 'CURRENT'),
+                build_version_object(2.7, 'v2', 'SUPPORTED'),
+                build_version_object(2.6, 'v2', 'SUPPORTED'),
                 build_version_object(2.5, 'v2', 'SUPPORTED'),
                 build_version_object(2.4, 'v2', 'SUPPORTED'),
                 build_version_object(2.3, 'v2', 'SUPPORTED'),
@@ -82,20 +97,6 @@ class Controller(object):
                 build_version_object(2.1, 'v2', 'SUPPORTED'),
                 build_version_object(2.0, 'v2', 'SUPPORTED'),
             ])
-        if CONF.enable_v1_api:
-            LOG.warn(_LW('The Images (Glance) v1 API is deprecated and will '
-                         'be removed on or after the Pike release, following '
-                         'the standard OpenStack deprecation policy. '
-                         'Currently, the solution is to set '
-                         'enable_v1_api=False and enable_v2_api=True in your '
-                         'glance-api.conf file. Once those options are '
-                         'removed from the code, Images (Glance) v2 API will '
-                         'be switched on by default and will be the only '
-                         'option to deploy and use.'))
-            version_objs.extend([
-                build_version_object(1.1, 'v1', 'DEPRECATED'),
-                build_version_object(1.0, 'v1', 'DEPRECATED'),
-            ])
 
         status = explicit and http_client.OK or http_client.MULTIPLE_CHOICES
         response = webob.Response(request=req,
