@@ -12,6 +12,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import functools
 import os
 
 import glance_store as store_api
@@ -20,7 +21,10 @@ from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
+from oslo_utils import timeutils
+from oslo_utils import units
 import six
+import taskflow
 from taskflow.patterns import linear_flow as lf
 from taskflow import retry
 from taskflow import task
@@ -82,6 +86,245 @@ class _NoStoresSucceeded(exception.GlanceException):
         super(_NoStoresSucceeded, self).__init__(message)
 
 
+class ImportActionWrapper(object):
+    """Wrapper for all the image metadata operations we do during an import.
+
+    This is used to consolidate the changes we make to image metadata during
+    an import operation, and can be used with an admin-capable repo to
+    enable non-owner controlled modification of that data if desired.
+
+    Use this as a context manager to make multiple changes followed by
+    a save of the image in one operation. An _ImportActions object is
+    yielded from the context manager, which defines the available operations.
+
+    :param image_repo: The ImageRepo we should use to fetch/save the image
+    :param image-id: The ID of the image we should be altering
+    """
+
+    def __init__(self, image_repo, image_id, task_id):
+        self._image_repo = image_repo
+        self._image_id = image_id
+        self._task_id = task_id
+
+    def __enter__(self):
+        self._image = self._image_repo.get(self._image_id)
+        self._image_previous_status = self._image.status
+        self._assert_task_lock(self._image)
+
+        return _ImportActions(self._image)
+
+    def __exit__(self, type, value, traceback):
+        if type is not None:
+            # NOTE(danms): Do not save the image if we raised in context
+            return
+
+        # NOTE(danms): If we were in the middle of a long-running
+        # set_data() where someone else stole our lock, we may race
+        # with them to update image locations and erase one that
+        # someone else is working on. Checking the task lock here
+        # again is not perfect exclusion, but in lieu of actual
+        # thread-safe location updating, this at least reduces the
+        # likelihood of that happening.
+        self.assert_task_lock()
+
+        if self._image_previous_status != self._image.status:
+            LOG.debug('Image %(image_id)s status changing from '
+                      '%(old_status)s to %(new_status)s',
+                      {'image_id': self._image_id,
+                       'old_status': self._image_previous_status,
+                       'new_status': self._image.status})
+        self._image_repo.save(self._image, self._image_previous_status)
+
+    @property
+    def image_id(self):
+        return self._image_id
+
+    def drop_lock_for_task(self):
+        """Delete the import lock for our task.
+
+        This is an atomic operation and thus does not require a context
+        for the image save. Note that after calling this method, no
+        further actions will be allowed on the image.
+
+        :raises: NotFound if the image was not locked by the expected task.
+        """
+        image = self._image_repo.get(self._image_id)
+        self._image_repo.delete_property_atomic(image,
+                                                'os_glance_import_task',
+                                                self._task_id)
+
+    def _assert_task_lock(self, image):
+        task_lock = image.extra_properties.get('os_glance_import_task')
+        if task_lock != self._task_id:
+            LOG.error('Image %(image)s import task %(task)s attempted to '
+                      'take action on image, but other task %(other)s holds '
+                      'the lock; Aborting.',
+                      {'image': self._image_id,
+                       'task': self._task_id,
+                       'other': task_lock})
+            raise exception.TaskAbortedError()
+
+    def assert_task_lock(self):
+        """Assert that we own the task lock on the image.
+
+        :raises: TaskAbortedError if we do not
+        """
+        image = self._image_repo.get(self._image_id)
+        self._assert_task_lock(image)
+
+
+class _ImportActions(object):
+    """Actions available for being performed on an image during import.
+
+    This defines the available actions that can be performed on an image
+    during import, which may be done with an image owned by another user.
+
+    Do not instantiate this object directly, get it from ImportActionWrapper.
+    """
+
+    IMPORTING_STORES_KEY = 'os_glance_importing_to_stores'
+    IMPORT_FAILED_KEY = 'os_glance_failed_import'
+
+    def __init__(self, image):
+        self._image = image
+
+    @property
+    def image_id(self):
+        return self._image.image_id
+
+    @property
+    def image_status(self):
+        return self._image.status
+
+    def merge_store_list(self, list_key, stores, subtract=False):
+        stores = set([store for store in stores if store])
+        existing = set(
+            self._image.extra_properties.get(list_key, '').split(','))
+
+        if subtract:
+            if stores - existing:
+                LOG.debug('Stores %(stores)s not in %(key)s for '
+                          'image %(image_id)s',
+                          {'stores': ','.join(sorted(stores - existing)),
+                           'key': list_key,
+                           'image_id': self.image_id})
+            merged_stores = existing - stores
+        else:
+            merged_stores = existing | stores
+
+        stores_list = ','.join(sorted((store for store in
+                                       merged_stores if store)))
+        self._image.extra_properties[list_key] = stores_list
+        LOG.debug('Image %(image_id)s %(key)s=%(stores)s',
+                  {'image_id': self.image_id,
+                   'key': list_key,
+                   'stores': stores_list})
+
+    def add_importing_stores(self, stores):
+        """Add a list of stores to the importing list.
+
+        Add stores to os_glance_importing_to_stores
+
+        :param stores: A list of store names
+        """
+        self.merge_store_list(self.IMPORTING_STORES_KEY, stores)
+
+    def remove_importing_stores(self, stores):
+        """Remove a list of stores from the importing list.
+
+        Remove stores from os_glance_importing_to_stores
+
+        :param stores: A list of store names
+        """
+        self.merge_store_list(self.IMPORTING_STORES_KEY, stores, subtract=True)
+
+    def add_failed_stores(self, stores):
+        """Add a list of stores to the failed list.
+
+        Add stores to os_glance_failed_import
+
+        :param stores: A list of store names
+        """
+        self.merge_store_list(self.IMPORT_FAILED_KEY, stores)
+
+    def remove_failed_stores(self, stores):
+        """Remove a list of stores from the failed list.
+
+        Remove stores from os_glance_failed_import
+
+        :param stores: A list of store names
+        """
+        self.merge_store_list(self.IMPORT_FAILED_KEY, stores, subtract=True)
+
+    def set_image_data(self, uri, task_id, backend, set_active,
+                       callback=None):
+        """Populate image with data on a specific backend.
+
+        This is used during an image import operation to populate the data
+        in a given store for the image. If this object wraps an admin-capable
+        image_repo, then this will be done with admin credentials on behalf
+        of a user already determined to be able to perform this operation
+        (such as a copy-image import of an existing image owned by another
+        user).
+
+        :param uri: Source URL for image data
+        :param task_id: The task responsible for this operation
+        :param backend: The backend store to target the data
+        :param set_active: Whether or not to set the image to 'active'
+                           state after the operation completes
+        :param callback: A callback function with signature:
+                         fn(action, chunk_bytes, total_bytes)
+                         which should be called while processing the image
+                         approximately every minute.
+        """
+        if callback:
+            callback = functools.partial(callback, self)
+        return image_import.set_image_data(self._image, uri, task_id,
+                                           backend=backend,
+                                           set_active=set_active,
+                                           callback=callback)
+
+    def set_image_status(self, status):
+        """Set the image status.
+
+        :param status: The new status of the image
+        """
+        self._image.status = status
+
+    def remove_location_for_store(self, backend):
+        """Remove a location from an image given a backend store.
+
+        Given a backend store, remove the corresponding location from the
+        image's set of locations. If the last location is removed, remove
+        the image checksum, hash information, and size.
+
+        :param backend: The backend store to remove from the image
+        """
+
+        for i, location in enumerate(self._image.locations):
+            if location.get('metadata', {}).get('store') == backend:
+                try:
+                    self._image.locations.pop(i)
+                except (store_exceptions.NotFound,
+                        store_exceptions.Forbidden):
+                    msg = (_("Error deleting from store %(store)s when "
+                             "reverting.") % {'store': backend})
+                    LOG.warning(msg)
+                # NOTE(yebinama): Some store drivers doesn't document which
+                # exceptions they throw.
+                except Exception:
+                    msg = (_("Unexpected exception when deleting from store "
+                             "%(store)s.") % {'store': backend})
+                    LOG.warning(msg)
+                else:
+                    if len(self._image.locations) == 0:
+                        self._image.checksum = None
+                        self._image.os_hash_algo = None
+                        self._image.os_hash_value = None
+                        self._image.size = None
+                break
+
+
 class _DeleteFromFS(task.Task):
 
     def __init__(self, task_id, task_type):
@@ -125,6 +368,41 @@ class _DeleteFromFS(task.Task):
                     'fn': file_path})
 
 
+class _ImageLock(task.Task):
+    def __init__(self, task_id, task_type, action_wrapper):
+        self.task_id = task_id
+        self.task_type = task_type
+        self.action_wrapper = action_wrapper
+        super(_ImageLock, self).__init__(
+            name='%s-ImageLock-%s' % (task_type, task_id))
+
+    def execute(self):
+        self.action_wrapper.assert_task_lock()
+        LOG.debug('Image %(image)s import task %(task)s lock confirmed',
+                  {'image': self.action_wrapper.image_id,
+                   'task': self.task_id})
+
+    def revert(self, result, **kwargs):
+        """Drop our claim on the image.
+
+        If we have failed, we need to drop our import_task lock on the image
+        so that something else can have a try. Note that we may have been
+        preempted so we should only drop *our* lock.
+        """
+        try:
+            self.action_wrapper.drop_lock_for_task()
+        except exception.NotFound:
+            LOG.warning('Image %(image)s import task %(task)s lost its '
+                        'lock during execution!',
+                        {'image': self.action_wrapper.image_id,
+                         'task': self.task_id})
+        else:
+            LOG.debug('Image %(image)s import task %(task)s dropped '
+                      'its lock after failure',
+                      {'image': self.action_wrapper.image_id,
+                       'task': self.task_id})
+
+
 class _VerifyStaging(task.Task):
 
     # NOTE(jokke): This could be also for example "staging_path" but to
@@ -201,23 +479,23 @@ class _VerifyStaging(task.Task):
 
 class _ImportToStore(task.Task):
 
-    def __init__(self, task_id, task_type, image_repo, uri, image_id, backend,
-                 all_stores_must_succeed, set_active):
+    def __init__(self, task_id, task_type, task_repo, action_wrapper, uri,
+                 backend, all_stores_must_succeed, set_active):
         self.task_id = task_id
         self.task_type = task_type
-        self.image_repo = image_repo
+        self.task_repo = task_repo
+        self.action_wrapper = action_wrapper
         self.uri = uri
-        self.image_id = image_id
         self.backend = backend
         self.all_stores_must_succeed = all_stores_must_succeed
         self.set_active = set_active
+        self.last_status = 0
         super(_ImportToStore, self).__init__(
             name='%s-ImportToStore-%s' % (task_type, task_id))
 
     def execute(self, file_path=None):
         """Bringing the imported image to back end store
 
-        :param image_id: Glance Image ID
         :param file_path: path to the image file
         """
         # NOTE(flaper87): Let's dance... and fall
@@ -260,14 +538,21 @@ class _ImportToStore(task.Task):
         # NOTE(jokke): The different options here are kind of pointless as we
         # will need the file path anyways for our delete workflow for now.
         # For future proofing keeping this as is.
-        image = self.image_repo.get(self.image_id)
-        if image.status == "deleted":
+
+        with self.action_wrapper as action:
+            self._execute(action, file_path)
+
+    def _execute(self, action, file_path):
+        self.last_status = timeutils.now()
+
+        if action.image_status == "deleted":
             raise exception.ImportTaskError("Image has been deleted, aborting"
                                             " import.")
         try:
-            image_import.set_image_data(image, file_path or self.uri,
-                                        self.task_id, backend=self.backend,
-                                        set_active=self.set_active)
+            action.set_image_data(file_path or self.uri,
+                                  self.task_id, backend=self.backend,
+                                  set_active=self.set_active,
+                                  callback=self._status_callback)
         # NOTE(yebinama): set_image_data catches Exception and raises from
         # them. Can't be more specific on exceptions catched.
         except Exception:
@@ -278,25 +563,32 @@ class _ImportToStore(task.Task):
                    {'task_id': self.task_id, 'task_type': self.task_type})
             LOG.warning(msg)
             if self.backend is not None:
-                failed_import = image.extra_properties.get(
-                    'os_glance_failed_import', '').split(',')
-                failed_import.append(self.backend)
-                image.extra_properties['os_glance_failed_import'] = ','.join(
-                    failed_import).lstrip(',')
+                action.add_failed_stores([self.backend])
+
         if self.backend is not None:
-            importing = image.extra_properties.get(
-                'os_glance_importing_to_stores', '').split(',')
-            try:
-                importing.remove(self.backend)
-                image.extra_properties[
-                    'os_glance_importing_to_stores'] = ','.join(
-                    importing).lstrip(',')
-            except ValueError:
-                LOG.debug("Store %s not found in property "
-                          "os_glance_importing_to_stores.", self.backend)
-        # NOTE(flaper87): We need to save the image again after
-        # the locations have been set in the image.
-        self.image_repo.save(image)
+            action.remove_importing_stores([self.backend])
+
+    def _status_callback(self, action, chunk_bytes, total_bytes):
+        # NOTE(danms): Only log status every five minutes
+        if timeutils.now() - self.last_status > 300:
+            LOG.debug('Image import %(image_id)s copied %(copied)i MiB',
+                      {'image_id': action.image_id,
+                       'copied': total_bytes // units.Mi})
+            self.last_status = timeutils.now()
+
+        task = script_utils.get_task(self.task_repo, self.task_id)
+        if task is None:
+            LOG.error(
+                'Status callback for task %(task)s found no task object!',
+                {'task': self.task_id})
+            raise exception.TaskNotFound(self.task_id)
+        if task.status != 'processing':
+            LOG.error('Task %(task)s expected "processing" status, '
+                      'but found "%(status)s"; aborting.')
+            raise exception.TaskAbortedError()
+
+        task.message = _('Copied %i MiB') % (total_bytes // units.Mi)
+        self.task_repo.save(task)
 
     def revert(self, result, **kwargs):
         """
@@ -304,40 +596,20 @@ class _ImportToStore(task.Task):
 
         :param result: taskflow result object
         """
-        image = self.image_repo.get(self.image_id)
-        for i, location in enumerate(image.locations):
-            if location.get('metadata', {}).get('store') == self.backend:
-                try:
-                    image.locations.pop(i)
-                except (store_exceptions.NotFound,
-                        store_exceptions.Forbidden):
-                    msg = (_("Error deleting from store %{store}s when "
-                             "reverting.") % {'store': self.backend})
-                    LOG.warning(msg)
-                # NOTE(yebinama): Some store drivers doesn't document which
-                # exceptions they throw.
-                except Exception:
-                    msg = (_("Unexpected exception when deleting from store"
-                             "%{store}s.") % {'store': self.backend})
-                    LOG.warning(msg)
-                else:
-                    if len(image.locations) == 0:
-                        image.checksum = None
-                        image.os_hash_algo = None
-                        image.os_hash_value = None
-                        image.size = None
-                    self.image_repo.save(image)
-                break
+        with self.action_wrapper as action:
+            action.remove_location_for_store(self.backend)
+            action.remove_importing_stores([self.backend])
+            if isinstance(result, taskflow.types.failure.Failure):
+                # We are the store that failed, so add us to the failed list
+                action.add_failed_stores([self.backend])
 
 
 class _VerifyImageState(task.Task):
 
-    def __init__(self, task_id, task_type, image_repo, image_id,
-                 import_method):
+    def __init__(self, task_id, task_type, action_wrapper, import_method):
         self.task_id = task_id
         self.task_type = task_type
-        self.image_repo = image_repo
-        self.image_id = image_id
+        self.action_wrapper = action_wrapper
         self.import_method = import_method
         super(_VerifyImageState, self).__init__(
             name='%s-VerifyImageState-%s' % (task_type, task_id))
@@ -347,38 +619,30 @@ class _VerifyImageState(task.Task):
 
         :param image_id: Glance Image ID
         """
-        new_image = self.image_repo.get(self.image_id)
-        if new_image.status != 'active':
-            raise _NoStoresSucceeded(_('None of the uploads finished!'))
+        with self.action_wrapper as action:
+            if action.image_status != 'active':
+                raise _NoStoresSucceeded(_('None of the uploads finished!'))
 
     def revert(self, result, **kwargs):
         """Set back to queued if this wasn't copy-image job."""
-        if self.import_method != 'copy-image':
-            new_image = self.image_repo.get(self.image_id)
-            new_image.status = 'queued'
-            self.image_repo.save_image(new_image)
+        with self.action_wrapper as action:
+            if self.import_method != 'copy-image':
+                action.set_image_status('queued')
 
 
 class _CompleteTask(task.Task):
 
-    def __init__(self, task_id, task_type, task_repo, image_id):
+    def __init__(self, task_id, task_type, task_repo, action_wrapper):
         self.task_id = task_id
         self.task_type = task_type
         self.task_repo = task_repo
-        self.image_id = image_id
+        self.action_wrapper = action_wrapper
         super(_CompleteTask, self).__init__(
             name='%s-CompleteTask-%s' % (task_type, task_id))
 
-    def execute(self):
-        """Finishing the task flow
-
-        :param image_id: Glance Image ID
-        """
-        task = script_utils.get_task(self.task_repo, self.task_id)
-        if task is None:
-            return
+    def _finish_task(self, task):
         try:
-            task.succeed({'image_id': self.image_id})
+            task.succeed({'image_id': self.action_wrapper.image_id})
         except Exception as e:
             # Note: The message string contains Error in it to indicate
             # in the task.message that it's a error message for the user.
@@ -397,6 +661,28 @@ class _CompleteTask(task.Task):
         finally:
             self.task_repo.save(task)
 
+    def _drop_lock(self):
+        try:
+            self.action_wrapper.drop_lock_for_task()
+        except exception.NotFound:
+            # NOTE(danms): This would be really bad, but there is probably
+            # not much point in reverting all the way back if we got this
+            # far. Log the carnage for forensics.
+            LOG.error('Image %(image)s import task %(task)s did not hold the '
+                      'lock upon completion!',
+                      {'image': self.action_wrapper.image_id,
+                       'task': self.task_id})
+
+    def execute(self):
+        """Finishing the task flow
+
+        :param image_id: Glance Image ID
+        """
+        task = script_utils.get_task(self.task_repo, self.task_id)
+        if task is not None:
+            self._finish_task(task)
+        self._drop_lock()
+
         LOG.info(_LI("%(task_id)s of %(task_type)s completed"),
                  {'task_id': self.task_id, 'task_type': self.task_type})
 
@@ -415,6 +701,7 @@ def get_flow(**kwargs):
     task_type = kwargs.get('task_type')
     task_repo = kwargs.get('task_repo')
     image_repo = kwargs.get('image_repo')
+    admin_repo = kwargs.get('admin_repo')
     image_id = kwargs.get('image_id')
     import_method = kwargs.get('import_req')['method']['name']
     uri = kwargs.get('import_req')['method'].get('uri')
@@ -426,6 +713,11 @@ def get_flow(**kwargs):
     if not CONF.enabled_backends and not CONF.node_staging_uri.endswith('/'):
         separator = '/'
 
+    # Instantiate an action wrapper with the admin repo if we got one,
+    # otherwise with the regular repo.
+    action_wrapper = ImportActionWrapper(admin_repo or image_repo, image_id,
+                                         task_id)
+
     if not uri and import_method in ['glance-direct', 'copy-image']:
         if CONF.enabled_backends:
             separator, staging_dir = store_utils.get_dir_separator()
@@ -435,6 +727,8 @@ def get_flow(**kwargs):
 
     flow = lf.Flow(task_type, retry=retry.AlwaysRevert())
 
+    flow.add(_ImageLock(task_id, task_type, action_wrapper))
+
     if import_method in ['web-download', 'copy-image']:
         internal_plugin = internal_plugins.get_import_plugin(**kwargs)
         flow.add(internal_plugin)
@@ -465,9 +759,9 @@ def get_flow(**kwargs):
         import_task = lf.Flow(task_name)
         import_to_store = _ImportToStore(task_id,
                                          task_name,
-                                         image_repo,
+                                         task_repo,
+                                         action_wrapper,
                                          file_uri,
-                                         image_id,
                                          store,
                                          all_stores_must_succeed,
                                          set_active)
@@ -479,27 +773,20 @@ def get_flow(**kwargs):
 
     verify_task = _VerifyImageState(task_id,
                                     task_type,
-                                    image_repo,
-                                    image_id,
+                                    action_wrapper,
                                     import_method)
     flow.add(verify_task)
 
     complete_task = _CompleteTask(task_id,
                                   task_type,
                                   task_repo,
-                                  image_id)
+                                  action_wrapper)
     flow.add(complete_task)
 
-    image = image_repo.get(image_id)
-    from_state = image.status
-    if import_method != 'copy-image':
-        image.status = 'importing'
-
-    image.extra_properties[
-        'os_glance_importing_to_stores'] = ','.join((store for store in
-                                                     stores if
-                                                     store is not None))
-    image.extra_properties['os_glance_failed_import'] = ''
-    image_repo.save(image, from_state=from_state)
+    with action_wrapper as action:
+        if import_method != 'copy-image':
+            action.set_image_status('importing')
+        action.add_importing_stores(stores)
+        action.remove_failed_stores(stores)
 
     return flow
