@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright 2011 OpenStack Foundation
+# Copyright 2018 RedHat Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -27,6 +27,7 @@ import functools
 import os
 import sys
 import time
+import uuid
 
 from oslo_utils import encodeutils
 import prettytable
@@ -50,6 +51,27 @@ SUCCESS = 0
 FAILURE = 1
 
 
+def validate_input(func):
+    """Decorator to enforce validation on input"""
+    @functools.wraps(func)
+    def wrapped(*args, **kwargs):
+        if len(args[0].command) > 2:
+            print("Please specify the ID of the image you wish for command "
+                  "'%s' from the cache as the first and only "
+                  "argument." % args[0].command[0])
+            return FAILURE
+        if len(args[0].command) == 2:
+            image_id = args[0].command[1]
+            try:
+                image_id = uuid.UUID(image_id)
+            except ValueError:
+                print("Image ID '%s' is not a valid UUID." % image_id)
+                return FAILURE
+
+        return func(args[0], **kwargs)
+    return wrapped
+
+
 def catch_error(action):
     """Decorator to provide sensible default error handling for actions."""
     def wrap(func):
@@ -145,18 +167,13 @@ def list_queued(args):
 
 
 @catch_error('queue the specified image for caching')
+@validate_input
 def queue_image(args):
     """%(prog)s queue-image <IMAGE_ID> [options]
 
     Queues an image for caching.
     """
-    if len(args.command) == 2:
-        image_id = args.command[1]
-    else:
-        print("Please specify one and only ID of the image you wish to ")
-        print("queue from the cache as the first argument")
-        return FAILURE
-
+    image_id = args.command[1]
     if (not args.force and
         not user_confirm("Queue image %(image_id)s for caching?" %
                          {'image_id': image_id}, default=False)):
@@ -173,18 +190,13 @@ def queue_image(args):
 
 
 @catch_error('delete the specified cached image')
+@validate_input
 def delete_cached_image(args):
     """%(prog)s delete-cached-image <IMAGE_ID> [options]
 
     Deletes an image from the cache.
     """
-    if len(args.command) == 2:
-        image_id = args.command[1]
-    else:
-        print("Please specify one and only ID of the image you wish to ")
-        print("delete from the cache as the first argument")
-        return FAILURE
-
+    image_id = args.command[1]
     if (not args.force and
         not user_confirm("Delete cached image %(image_id)s?" %
                          {'image_id': image_id}, default=False)):
@@ -220,18 +232,13 @@ def delete_all_cached_images(args):
 
 
 @catch_error('delete the specified queued image')
+@validate_input
 def delete_queued_image(args):
     """%(prog)s delete-queued-image <IMAGE_ID> [options]
 
     Deletes an image from the cache.
     """
-    if len(args.command) == 2:
-        image_id = args.command[1]
-    else:
-        print("Please specify one and only ID of the image you wish to ")
-        print("delete from the cache as the first argument")
-        return FAILURE
-
+    image_id = args.command[1]
     if (not args.force and
         not user_confirm("Delete queued image %(image_id)s?" %
                          {'image_id': image_id}, default=False)):
@@ -272,13 +279,30 @@ def get_client(options):
     specified by the --host and --port options
     supplied to the CLI
     """
+    # Generate auth_url based on identity_api_version
+    identity_version = env('OS_IDENTITY_API_VERSION', default='3')
+    auth_url = options.os_auth_url
+    if identity_version == '3' and "/v3" not in auth_url:
+        auth_url = auth_url + "/v3"
+    elif identity_version == '2' and "/v2" not in auth_url:
+        auth_url = auth_url + "/v2.0"
+
+    user_domain_id = options.os_user_domain_id
+    if not user_domain_id:
+        user_domain_id = options.os_domain_id
+    project_domain_id = options.os_project_domain_id
+    if not user_domain_id:
+        project_domain_id = options.os_domain_id
+
     return glance.image_cache.client.get_client(
         host=options.host,
         port=options.port,
         username=options.os_username,
         password=options.os_password,
-        tenant=options.os_tenant_name,
-        auth_url=options.os_auth_url,
+        project=options.os_project_name,
+        user_domain_id=user_domain_id,
+        project_domain_id=project_domain_id,
+        auth_url=auth_url,
         auth_strategy=options.os_auth_strategy,
         auth_token=options.os_auth_token,
         region=options.os_region_name,
@@ -312,7 +336,7 @@ def parse_args(parser):
 
     :param parser: The option parser
     """
-    parser.add_argument('command', default='help', nargs='*',
+    parser.add_argument('command', default='help', nargs='+',
                         help='The command to execute')
     parser.add_argument('-v', '--verbose', default=False, action="store_true",
                         help="Print more verbose output.")
@@ -367,22 +391,36 @@ def parse_args(parser):
                         dest='os_region_name',
                         help=argparse.SUPPRESS)
 
-    parser.add_argument('--os-tenant-id',
-                        dest='os_tenant_id',
-                        default=env('OS_TENANT_ID'),
-                        help='Defaults to env[OS_TENANT_ID].')
-    parser.add_argument('--os_tenant_id',
-                        dest='os_tenant_id',
+    parser.add_argument('--os-project-id',
+                        dest='os_project_id',
+                        default=env('OS_PROJECT_ID'),
+                        help='Defaults to env[OS_PROJECT_ID].')
+    parser.add_argument('--os_project_id',
+                        dest='os_project_id',
                         help=argparse.SUPPRESS)
 
-    parser.add_argument('--os-tenant-name',
-                        dest='os_tenant_name',
-                        default=env('OS_TENANT_NAME'),
-                        help='Defaults to env[OS_TENANT_NAME].')
-    parser.add_argument('-T', '--os_tenant_name',
-                        dest='os_tenant_name',
+    parser.add_argument('--os-project-name',
+                        dest='os_project_name',
+                        default=env('OS_PROJECT_NAME'),
+                        help='Defaults to env[OS_PROJECT_NAME].')
+    parser.add_argument('-T', '--os_project_name',
+                        dest='os_project_name',
                         help=argparse.SUPPRESS)
 
+    # arguments related user, project domain
+    parser.add_argument('--os-user-domain-id',
+                        dest='os_user_domain_id',
+                        default=env('OS_USER_DOMAIN_ID'),
+                        help='Defaults to env[OS_USER_DOMAIN_ID].')
+    parser.add_argument('--os-project-domain-id',
+                        dest='os_project_domain_id',
+                        default=env('OS_PROJECT_DOMAIN_ID'),
+                        help='Defaults to env[OS_PROJECT_DOMAIN_ID].')
+    parser.add_argument('--os-domain-id',
+                        dest='os_domain_id',
+                        default=env('OS_DOMAIN_ID', default='default'),
+                        help='Defaults to env[OS_DOMAIN_ID].')
+
     parser.add_argument('--os-auth-url',
                         default=env('OS_AUTH_URL'),
                         help='Defaults to env[OS_AUTH_URL].')
@@ -486,5 +524,6 @@ def main():
     except (RuntimeError, NotImplementedError) as e:
         sys.exit("ERROR: %s" % e)
 
+
 if __name__ == '__main__':
     main()
