@@ -39,6 +39,13 @@ if os.name == 'nt':
 else:
     eventlet.patcher.monkey_patch()
 
+# Monkey patch the original current_thread to use the up-to-date _active
+# global variable. See https://bugs.launchpad.net/bugs/1863021 and
+# https://github.com/eventlet/eventlet/issues/592
+import __original_module_threading as orig_threading
+import threading
+orig_threading.current_thread.__globals__['_active'] = threading._active
+
 from oslo_reports import guru_meditation_report as gmr
 from oslo_utils import encodeutils
 
