@@ -18,18 +18,21 @@ This auth module is intended to allow OpenStack client-tools to select from a
 variety of authentication strategies, including NoAuth (the default), and
 Keystone (an identity management system).
 
-    > auth_plugin = AuthPlugin(creds)
+::
 
-    > auth_plugin.authenticate()
+   > auth_plugin = AuthPlugin(creds)
 
-    > auth_plugin.auth_token
+   > auth_plugin.authenticate()
+
+   > auth_plugin.auth_token
     abcdefg
 
-    > auth_plugin.management_url
-    http://service_endpoint/
+   > auth_plugin.management_url
+   http://service_endpoint/
+
 """
 import httplib2
-from keystoneauth1.access import service_catalog as ks_service_catalog
+from keystoneclient import service_catalog as ks_service_catalog
 from oslo_serialization import jsonutils
 from six.moves import http_client as http
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
@@ -323,10 +326,11 @@ def get_endpoint(service_catalog, service_type='image', endpoint_region=None,
     otherwise we will raise an exception.
     """
     endpoints = ks_service_catalog.ServiceCatalogV2(
-        service_catalog).get_urls(interface=endpoint_type,
-                                  service_type=service_type,
-                                  region_name=endpoint_region)
-    if len(endpoints) == 0:
+        {'serviceCatalog': service_catalog}
+    ).get_urls(service_type=service_type,
+               region_name=endpoint_region,
+               endpoint_type=endpoint_type)
+    if endpoints is None:
         raise exception.NoServiceEndpoint()
     elif len(endpoints) == 1:
         return endpoints[0]
