@@ -171,6 +171,13 @@ _DEPRECATE_GLANCE_V1_MSG = _('The Images (Glance) version 1 API has been '
 
 common_opts = [
     cfg.BoolOpt('allow_additional_image_properties', default=True,
+                deprecated_for_removal=True,
+                deprecated_since="Ussuri",
+                deprecated_reason=_("""
+This option is redundant.  Control custom image property usage via the
+'image_property_quota' configuration option.  This option is scheduled
+to be removed during the Victoria development cycle.
+"""),
                 help=_("""
 Allow users to add additional/custom properties to images.
 
@@ -684,14 +691,15 @@ Related options:
     cfg.ListOpt('enabled_import_methods',
                 item_type=cfg.types.String(quotes=True),
                 bounds=True,
-                default=['glance-direct', 'web-download'],
+                default=['glance-direct', 'web-download',
+                         'copy-image'],
                 help=_("""
-List of enabled Image Import Methods
+    List of enabled Image Import Methods
 
-Both 'glance-direct' and 'web-download' are enabled by default.
+    'glance-direct', 'copy-image' and 'web-download' are enabled by default.
 
-Related options:
-    * [DEFAULT]/node_staging_uri""")),
+    Related options:
+        * [DEFAULT]/node_staging_uri""")),
 ]
 
 CONF = cfg.CONF
