@@ -29,14 +29,12 @@ except ImportError:
 from eventlet.green import socket
 
 import functools
+import glance_store
 import os
 import re
-import uuid
 
-from OpenSSL import crypto
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import netutils
 from oslo_utils import strutils
@@ -46,7 +44,8 @@ from webob import exc
 
 from glance.common import exception
 from glance.common import timeutils
-from glance.i18n import _, _LE, _LW
+from glance.common import wsgi
+from glance.i18n import _, _LE
 
 CONF = cfg.CONF
 
@@ -440,61 +439,6 @@ def setup_remote_pydev_debug(host, port):
             LOG.exception(error_msg)
 
 
-def validate_key_cert(key_file, cert_file):
-    try:
-        error_key_name = "private key"
-        error_filename = key_file
-        with open(key_file, 'r') as keyfile:
-            key_str = keyfile.read()
-        key = crypto.load_privatekey(crypto.FILETYPE_PEM, key_str)
-
-        error_key_name = "certificate"
-        error_filename = cert_file
-        with open(cert_file, 'r') as certfile:
-            cert_str = certfile.read()
-        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_str)
-    except IOError as ioe:
-        raise RuntimeError(_("There is a problem with your %(error_key_name)s "
-                             "%(error_filename)s.  Please verify it."
-                             "  Error: %(ioe)s") %
-                           {'error_key_name': error_key_name,
-                            'error_filename': error_filename,
-                            'ioe': ioe})
-    except crypto.Error as ce:
-        raise RuntimeError(_("There is a problem with your %(error_key_name)s "
-                             "%(error_filename)s.  Please verify it. OpenSSL"
-                             " error: %(ce)s") %
-                           {'error_key_name': error_key_name,
-                            'error_filename': error_filename,
-                            'ce': ce})
-
-    try:
-        data = str(uuid.uuid4())
-        # On Python 3, explicitly encode to UTF-8 to call crypto.sign() which
-        # requires bytes. Otherwise, it raises a deprecation warning (and
-        # will raise an error later).
-        data = encodeutils.to_utf8(data)
-        digest = CONF.digest_algorithm
-        if digest == 'sha1':
-            LOG.warn(
-                _LW('The FIPS (FEDERAL INFORMATION PROCESSING STANDARDS)'
-                    ' state that the SHA-1 is not suitable for'
-                    ' general-purpose digital signature applications (as'
-                    ' specified in FIPS 186-3) that require 112 bits of'
-                    ' security. The default value is sha1 in Kilo for a'
-                    ' smooth upgrade process, and it will be updated'
-                    ' with sha256 in next release(L).'))
-        out = crypto.sign(key, data, digest)
-        crypto.verify(cert, out, data, digest)
-    except crypto.Error as ce:
-        raise RuntimeError(_("There is a problem with your key pair.  "
-                             "Please verify that cert %(cert_file)s and "
-                             "key %(key_file)s belong together.  OpenSSL "
-                             "error %(ce)s") % {'cert_file': cert_file,
-                                                'key_file': key_file,
-                                                'ce': ce})
-
-
 def get_test_suite_socket():
     global GLANCE_TEST_SOCKET_FD_STR
     if GLANCE_TEST_SOCKET_FD_STR in os.environ:
@@ -616,14 +560,6 @@ def stash_conf_values():
         'backlog': CONF.backlog,
     }
 
-    # Note(jokke): As we can still support termination with PY27 we
-    # need to add the possible certs in that specific case. This is
-    # only needed prior Ussuri which doesn't support PY27 anymore.
-    if six.PY2:
-        conf['key_file'] = CONF.key_file,
-        conf['cert_file'] = CONF.cert_file
-        conf['tcp_keepidle'] = CONF.cert_file,
-
     return conf
 
 
@@ -731,3 +667,50 @@ def evaluate_filter_op(value, operator, threshold):
 
     msg = _("Unable to filter on a unknown operator.")
     raise exception.InvalidFilterOperatorValue(msg)
+
+
+def _get_available_stores():
+    available_stores = CONF.enabled_backends
+    stores = []
+    # Remove reserved stores from the available stores list
+    for store in available_stores:
+        # NOTE (abhishekk): http store is readonly and should be
+        # excluded from the list.
+        if available_stores[store] == 'http':
+            continue
+        if store not in wsgi.RESERVED_STORES:
+            stores.append(store)
+
+    return stores
+
+
+def get_stores_from_request(req, body):
+    """Processes a supplied request and extract stores from it
+
+    :param req: request to process
+    :param body: request body
+
+    :raises glance_store.UnknownScheme:  if a store is not valid
+    :return: a list of stores
+    """
+    if body.get('all_stores', False):
+        if 'stores' in body or 'x-image-meta-store' in req.headers:
+            msg = _("All_stores parameter can't be used with "
+                    "x-image-meta-store header or stores parameter")
+            raise exc.HTTPBadRequest(explanation=msg)
+        stores = _get_available_stores()
+    else:
+        try:
+            stores = body['stores']
+        except KeyError:
+            stores = [req.headers.get('x-image-meta-store',
+                                      CONF.glance_store.default_backend)]
+        else:
+            if 'x-image-meta-store' in req.headers:
+                msg = _("Stores parameter and x-image-meta-store header can't "
+                        "be both specified")
+                raise exc.HTTPBadRequest(explanation=msg)
+    # Validate each store
+    for store in stores:
+        glance_store.get_store_from_store_identifier(store)
+    return stores
