@@ -71,6 +71,7 @@ IMAGE_ATTRS = BASE_MODEL_ATTRS | set(['name', 'status', 'size', 'virtual_size',
                                       'min_disk', 'min_ram', 'is_public',
                                       'locations', 'checksum', 'owner',
                                       'protected'])
+IMAGE_ATOMIC_PROPS = set(['os_glance_import_task'])
 
 
 class ImageRepo(object):
@@ -130,13 +131,16 @@ class ImageRepo(object):
             protected=db_image['protected'],
             locations=location_strategy.get_ordered_locations(locations),
             checksum=db_image['checksum'],
+            os_hash_algo=db_image['os_hash_algo'],
+            os_hash_value=db_image['os_hash_value'],
             owner=db_image['owner'],
             disk_format=db_image['disk_format'],
             container_format=db_image['container_format'],
             size=db_image['size'],
             virtual_size=db_image['virtual_size'],
             extra_properties=properties,
-            tags=db_tags
+            tags=db_tags,
+            os_hidden=db_image['os_hidden'],
         )
 
     def _format_image_to_db(self, image):
@@ -161,6 +165,8 @@ class ImageRepo(object):
             'protected': image.protected,
             'locations': locations,
             'checksum': image.checksum,
+            'os_hash_algo': image.os_hash_algo,
+            'os_hash_value': image.os_hash_value,
             'owner': image.owner,
             'disk_format': image.disk_format,
             'container_format': image.container_format,
@@ -168,6 +174,7 @@ class ImageRepo(object):
             'virtual_size': image.virtual_size,
             'visibility': image.visibility,
             'properties': dict(image.extra_properties),
+            'os_hidden': image.os_hidden
         }
 
     def add(self, image):
@@ -194,7 +201,9 @@ class ImageRepo(object):
                                                   image.image_id,
                                                   image_values,
                                                   purge_props=True,
-                                                  from_state=from_state)
+                                                  from_state=from_state,
+                                                  atomic_props=(
+                                                      IMAGE_ATOMIC_PROPS))
         except (exception.ImageNotFound, exception.Forbidden):
             msg = _("No image found with ID %s") % image.image_id
             raise exception.ImageNotFound(msg)
@@ -214,6 +223,14 @@ class ImageRepo(object):
         new_values = self.db_api.image_destroy(self.context, image.image_id)
         image.updated_at = new_values['updated_at']
 
+    def set_property_atomic(self, image, name, value):
+        self.db_api.image_set_property_atomic(
+            image.image_id, name, value)
+
+    def delete_property_atomic(self, image, name, value):
+        self.db_api.image_delete_property_atomic(
+            image.image_id, name, value)
+
 
 class ImageProxy(glance.domain.proxy.Image):
 
