@@ -426,11 +426,39 @@ def _sort_images(images, sort_key, sort_dir):
     return images
 
 
+def image_set_property_atomic(image_id, name, value):
+    try:
+        image = DATA['images'][image_id]
+    except KeyError:
+        LOG.warn(_LW('Could not find image %s'), image_id)
+        raise exception.ImageNotFound()
+
+    prop = _image_property_format(image_id,
+                                  name,
+                                  value)
+    image['properties'].append(prop)
+
+
+def image_delete_property_atomic(image_id, name, value):
+    try:
+        image = DATA['images'][image_id]
+    except KeyError:
+        LOG.warn(_LW('Could not find image %s'), image_id)
+        raise exception.ImageNotFound()
+
+    for i, prop in enumerate(image['properties']):
+        if prop['name'] == name and prop['value'] == value:
+            del image['properties'][i]
+            return
+
+    raise exception.NotFound()
+
+
 def _image_get(context, image_id, force_show_deleted=False, status=None):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.warn(_LW('Could not find image %s') % image_id)
+        LOG.warn(_LW('Could not find image %s'), image_id)
         raise exception.ImageNotFound()
 
     if image['deleted'] and not (force_show_deleted
@@ -756,7 +784,7 @@ def image_create(context, image_values, v1_mode=False):
 
 @log_call
 def image_update(context, image_id, image_values, purge_props=False,
-                 from_state=None, v1_mode=False):
+                 from_state=None, v1_mode=False, atomic_props=None):
     global DATA
     try:
         image = DATA['images'][image_id]
@@ -767,17 +795,24 @@ def image_update(context, image_id, image_values, purge_props=False,
     if location_data is not None:
         _image_locations_set(context, image_id, location_data)
 
+    if atomic_props is None:
+        atomic_props = []
+
     # replace values for properties that already exist
     new_properties = image_values.pop('properties', {})
     for prop in image['properties']:
-        if prop['name'] in new_properties:
+        if prop['name'] in atomic_props:
+            continue
+        elif prop['name'] in new_properties:
             prop['value'] = new_properties.pop(prop['name'])
         elif purge_props:
             # this matches weirdness in the sqlalchemy api
             prop['deleted'] = True
 
     image['updated_at'] = timeutils.utcnow()
-    _image_update(image, image_values, new_properties)
+    _image_update(image, image_values,
+                  {k: v for k, v in new_properties.items()
+                   if k not in atomic_props})
     DATA['images'][image_id] = image
 
     image = _normalize_locations(context, copy.deepcopy(image))
@@ -1056,8 +1091,11 @@ def _sort_tasks(tasks, sort_key, sort_dir):
     reverse = False
     if tasks and not (sort_key in tasks[0]):
         raise exception.InvalidSortKey()
-    keyfn = lambda x: (x[sort_key] if x[sort_key] is not None else '',
-                       x['created_at'], x['id'])
+
+    def keyfn(x):
+        return (x[sort_key] if x[sort_key] is not None else '',
+                x['created_at'], x['id'])
+
     reverse = sort_dir == 'desc'
     tasks.sort(key=keyfn, reverse=reverse)
     return tasks
