@@ -119,7 +119,9 @@ class Image(BASE, GlanceBase):
                       Index('ix_images_deleted', 'deleted'),
                       Index('owner_image_idx', 'owner'),
                       Index('created_at_image_idx', 'created_at'),
-                      Index('updated_at_image_idx', 'updated_at'))
+                      Index('updated_at_image_idx', 'updated_at'),
+                      Index('os_hidden_image_idx', 'os_hidden'),
+                      Index('os_hash_value_image_idx', 'os_hash_value'))
 
     id = Column(String(36), primary_key=True,
                 default=lambda: str(uuid.uuid4()))
@@ -133,11 +135,15 @@ class Image(BASE, GlanceBase):
                         name='image_visibility'), nullable=False,
                         server_default='shared')
     checksum = Column(String(32))
+    os_hash_algo = Column(String(64))
+    os_hash_value = Column(String(128))
     min_disk = Column(Integer, nullable=False, default=0)
     min_ram = Column(Integer, nullable=False, default=0)
     owner = Column(String(255))
     protected = Column(Boolean, nullable=False, default=False,
                        server_default=sql.expression.false())
+    os_hidden = Column(Boolean, nullable=False, default=False,
+                       server_default=sql.expression.false())
 
 
 class ImageProperty(BASE, GlanceBase):
