@@ -34,9 +34,13 @@ import socket
 import subprocess
 import sys
 import tempfile
+import textwrap
 import time
+from unittest import mock
+import uuid
 
 import fixtures
+import glance_store
 from os_win import utilsfactory as os_win_utilsfactory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
@@ -44,8 +48,11 @@ from oslo_serialization import jsonutils
 from six.moves import range
 import six.moves.urllib.parse as urlparse
 import testtools
+import webob
 
+from glance.common import config
 from glance.common import utils
+from glance.common import wsgi
 from glance.db.sqlalchemy import api as db_api
 from glance import tests as glance_tests
 from glance.tests import utils as test_utils
@@ -60,7 +67,6 @@ else:
 
 
 CONF = cfg.CONF
-CONF.import_opt('registry_host', 'glance.registry')
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -90,7 +96,6 @@ class BaseServer(object):
         self.show_multiple_locations = False
         self.property_protection_file = ''
         self.enable_v2_api = True
-        self.enable_v2_registry = True
         self.needs_database = False
         self.log_file = None
         self.sock = sock
@@ -378,9 +383,6 @@ class ApiServer(Server):
         self.server_module = 'glance.cmd.%s' % self.server_name
         self.default_store = kwargs.get("default_store", "file")
         self.bind_host = "127.0.0.1"
-        self.registry_host = "127.0.0.1"
-        self.key_file = ""
-        self.cert_file = ""
         self.metadata_encryption_key = "012345678901234567890123456789ab"
         self.image_dir = os.path.join(self.test_dir, "images")
         self.pid_file = pid_file or os.path.join(self.test_dir, "api.pid")
@@ -416,16 +418,15 @@ class ApiServer(Server):
 
         self.send_identity_headers = False
 
+        self.node_staging_uri = 'file://%s' % os.path.join(
+            self.test_dir, 'staging')
+
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
 default_log_levels = eventlet.wsgi.server=DEBUG
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
-key_file = %(key_file)s
-cert_file = %(cert_file)s
 metadata_encryption_key = %(metadata_encryption_key)s
-registry_host = %(registry_host)s
-registry_port = %(registry_port)s
 use_user_token = %(use_user_token)s
 send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
@@ -452,6 +453,7 @@ image_tag_quota=%(image_tag_quota)s
 image_location_quota=%(image_location_quota)s
 location_strategy=%(location_strategy)s
 allow_additional_image_properties = True
+node_staging_uri=%(node_staging_uri)s
 [oslo_policy]
 policy_file = %(policy_file)s
 policy_default_rule = %(policy_default_rule)s
@@ -559,12 +561,10 @@ class ApiServerForMultipleBackend(Server):
         self.server_module = 'glance.cmd.%s' % self.server_name
         self.default_backend = kwargs.get("default_backend", "file1")
         self.bind_host = "127.0.0.1"
-        self.registry_host = "127.0.0.1"
-        self.key_file = ""
-        self.cert_file = ""
         self.metadata_encryption_key = "012345678901234567890123456789ab"
         self.image_dir_backend_1 = os.path.join(self.test_dir, "images_1")
         self.image_dir_backend_2 = os.path.join(self.test_dir, "images_2")
+        self.image_dir_backend_3 = os.path.join(self.test_dir, "images_3")
         self.staging_dir = os.path.join(self.test_dir, "staging")
         self.pid_file = pid_file or os.path.join(self.test_dir,
                                                  "multiple_backend_api.pid")
@@ -605,11 +605,7 @@ debug = %(debug)s
 default_log_levels = eventlet.wsgi.server=DEBUG
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
-key_file = %(key_file)s
-cert_file = %(cert_file)s
 metadata_encryption_key = %(metadata_encryption_key)s
-registry_host = %(registry_host)s
-registry_port = %(registry_port)s
 use_user_token = %(use_user_token)s
 send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
@@ -636,7 +632,7 @@ image_tag_quota=%(image_tag_quota)s
 image_location_quota=%(image_location_quota)s
 location_strategy=%(location_strategy)s
 allow_additional_image_properties = True
-enabled_backends=file1:file,file2:file
+enabled_backends=file1:file,file2:file,file3:file
 [oslo_policy]
 policy_file = %(policy_file)s
 policy_default_rule = %(policy_default_rule)s
@@ -650,6 +646,8 @@ default_backend = %(default_backend)s
 filesystem_store_datadir=%(image_dir_backend_1)s
 [file2]
 filesystem_store_datadir=%(image_dir_backend_2)s
+[file3]
+filesystem_store_datadir=%(image_dir_backend_3)s
 [import_filtering_opts]
 allowed_ports = []
 [os_glance_staging_store]
@@ -735,88 +733,6 @@ allowed_origin=http://valid.example.com
 """
 
 
-class RegistryServer(Server):
-
-    """
-    Server object that starts/stops/manages the Registry server
-    """
-
-    def __init__(self, test_dir, port, policy_file, sock=None):
-        super(RegistryServer, self).__init__(test_dir, port, sock=sock)
-        self.server_name = 'registry'
-        self.server_module = 'glance.cmd.%s' % self.server_name
-
-        self.needs_database = True
-        default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
-        self.sql_connection = os.environ.get('GLANCE_TEST_SQL_CONNECTION',
-                                             default_sql_connection)
-
-        self.bind_host = "127.0.0.1"
-        self.pid_file = os.path.join(self.test_dir, "registry.pid")
-        self.log_file = os.path.join(self.test_dir, "registry.log")
-        self.owner_is_tenant = True
-        self.workers = 0
-        self.api_version = 1
-        self.user_storage_quota = '0'
-        self.metadata_encryption_key = "012345678901234567890123456789ab"
-        self.policy_file = policy_file
-        self.policy_default_rule = 'default'
-        self.disable_path = None
-        self.image_cache_dir = os.path.join(self.test_dir,
-                                            'cache')
-        self.image_cache_driver = 'sqlite'
-
-        self.conf_base = """[DEFAULT]
-debug = %(debug)s
-image_cache_dir = %(image_cache_dir)s
-image_cache_driver = %(image_cache_driver)s
-bind_host = %(bind_host)s
-bind_port = %(bind_port)s
-log_file = %(log_file)s
-sql_connection = %(sql_connection)s
-sql_idle_timeout = 3600
-api_limit_max = 1000
-limit_param_default = 25
-owner_is_tenant = %(owner_is_tenant)s
-enable_v2_registry = %(enable_v2_registry)s
-workers = %(workers)s
-user_storage_quota = %(user_storage_quota)s
-metadata_encryption_key = %(metadata_encryption_key)s
-[oslo_policy]
-policy_file = %(policy_file)s
-policy_default_rule = %(policy_default_rule)s
-[paste_deploy]
-flavor = %(deployment_flavor)s
-"""
-        self.paste_conf_base = """[pipeline:glance-registry]
-pipeline = healthcheck unauthenticated-context registryapp
-
-[pipeline:glance-registry-fakeauth]
-pipeline = healthcheck fakeauth context registryapp
-
-[pipeline:glance-registry-trusted-auth]
-pipeline = healthcheck context registryapp
-
-[app:registryapp]
-paste.app_factory = glance.registry.api:API.factory
-
-[filter:healthcheck]
-paste.filter_factory = oslo_middleware:Healthcheck.factory
-backends = disable_by_file
-disable_by_file_path = %(disable_path)s
-
-[filter:context]
-paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory
-
-[filter:unauthenticated-context]
-paste.filter_factory =
- glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
-
-[filter:fakeauth]
-paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
-"""
-
-
 class ScrubberDaemon(Server):
     """
     Server object that starts/stops/manages the Scrubber server
@@ -829,8 +745,6 @@ class ScrubberDaemon(Server):
         self.server_module = 'glance.cmd.%s' % self.server_name
         self.daemon = daemon
 
-        self.registry_host = "127.0.0.1"
-
         self.image_dir = os.path.join(self.test_dir, "images")
         self.scrub_time = 5
         self.pid_file = os.path.join(self.test_dir, "scrubber.pid")
@@ -890,7 +804,6 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
-        self.registry_port, reg_sock = test_utils.get_unused_port_and_socket()
         # NOTE: Scrubber is enabled by default for the functional tests.
         # Please disbale it by explicitly setting 'self.include_scrubber' to
         # False in the test SetUps that do not require Scrubber to run.
@@ -898,8 +811,7 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         # The clients will try to connect to this address. Let's make sure
         # we're not using the default '0.0.0.0'
-        self.config(bind_host='127.0.0.1',
-                    registry_host='127.0.0.1')
+        self.config(bind_host='127.0.0.1')
 
         self.tracecmd = tracecmd_osmap.get(platform.system())
 
@@ -921,15 +833,9 @@ class FunctionalTest(test_utils.BaseTestCase):
                                     self.policy_file,
                                     sock=api_sock)
 
-        self.registry_server = RegistryServer(self.test_dir,
-                                              self.registry_port,
-                                              self.policy_file,
-                                              sock=reg_sock)
-
         self.scrubber_daemon = ScrubberDaemon(self.test_dir, self.policy_file)
 
         self.pid_files = [self.api_server.pid_file,
-                          self.registry_server.pid_file,
                           self.scrubber_daemon.pid_file]
         self.files_to_destroy = []
         self.launched_servers = []
@@ -941,12 +847,9 @@ class FunctionalTest(test_utils.BaseTestCase):
             # We destroy the test data store between each test case,
             # and recreate it, which ensures that we have no side-effects
             # from the tests
-            self.addCleanup(
-                self._reset_database, self.registry_server.sql_connection)
             self.addCleanup(
                 self._reset_database, self.api_server.sql_connection)
             self.addCleanup(self.cleanup)
-            self._reset_database(self.registry_server.sql_connection)
             self._reset_database(self.api_server.sql_connection)
 
     def set_policy_rules(self, rules):
@@ -995,7 +898,6 @@ class FunctionalTest(test_utils.BaseTestCase):
         # between a child process listening on a port actually dying
         # and a new process being started
         servers = [self.api_server,
-                   self.registry_server,
                    self.scrubber_daemon]
         for s in servers:
             try:
@@ -1078,7 +980,7 @@ class FunctionalTest(test_utils.BaseTestCase):
     def start_servers(self, **kwargs):
         """
         Starts the API and Registry servers (glance-control api start
-        & glance-control registry start) on unused ports.  glance-control
+        ) on unused ports.  glance-control
         should be installed into the python path
 
         Any kwargs passed to this method will override the configuration
@@ -1086,14 +988,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         """
         self.cleanup()
 
-        # Start up the API and default registry server
-
-        # We start the registry server first, as the API server config
-        # depends on the registry port - this ordering allows for
-        # retrying the launch on a port clash
-        self.start_with_retry(self.registry_server, 'registry_port', 3,
-                              **kwargs)
-        kwargs['registry_port'] = self.registry_server.bind_port
+        # Start up the API server
 
         self.start_with_retry(self.api_server, 'api_port', 3, **kwargs)
 
@@ -1214,14 +1109,11 @@ class FunctionalTest(test_utils.BaseTestCase):
         is meant to be called during a normal test case sequence.
         """
 
-        # Spin down the API and default registry server
+        # Spin down the API server
         self.stop_server(self.api_server)
-        self.stop_server(self.registry_server)
         if self.include_scrubber:
             self.stop_server(self.scrubber_daemon)
 
-        self._reset_database(self.registry_server.sql_connection)
-
     def run_sql_cmd(self, sql):
         """
         Provides a crude mechanism to run manual SQL commands for backend
@@ -1265,7 +1157,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
 
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
-        self.registry_port, reg_sock = test_utils.get_unused_port_and_socket()
         # NOTE: Scrubber is enabled by default for the functional tests.
         # Please disbale it by explicitly setting 'self.include_scrubber' to
         # False in the test SetUps that do not require Scrubber to run.
@@ -1289,15 +1180,9 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         self.api_server_multiple_backend = ApiServerForMultipleBackend(
             self.test_dir, self.api_port, self.policy_file, sock=api_sock)
 
-        self.registry_server = RegistryServer(self.test_dir,
-                                              self.registry_port,
-                                              self.policy_file,
-                                              sock=reg_sock)
-
         self.scrubber_daemon = ScrubberDaemon(self.test_dir, self.policy_file)
 
         self.pid_files = [self.api_server_multiple_backend.pid_file,
-                          self.registry_server.pid_file,
                           self.scrubber_daemon.pid_file]
         self.files_to_destroy = []
         self.launched_servers = []
@@ -1309,13 +1194,10 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
             # We destroy the test data store between each test case,
             # and recreate it, which ensures that we have no side-effects
             # from the tests
-            self.addCleanup(
-                self._reset_database, self.registry_server.sql_connection)
             self.addCleanup(
                 self._reset_database,
                 self.api_server_multiple_backend.sql_connection)
             self.addCleanup(self.cleanup)
-            self._reset_database(self.registry_server.sql_connection)
             self._reset_database(
                 self.api_server_multiple_backend.sql_connection)
 
@@ -1365,7 +1247,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         # between a child process listening on a port actually dying
         # and a new process being started
         servers = [self.api_server_multiple_backend,
-                   self.registry_server,
                    self.scrubber_daemon]
         for s in servers:
             try:
@@ -1448,7 +1329,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
     def start_servers(self, **kwargs):
         """
         Starts the API and Registry servers (glance-control api start
-        & glance-control registry start) on unused ports.  glance-control
+        ) on unused ports.  glance-control
         should be installed into the python path
 
         Any kwargs passed to this method will override the configuration
@@ -1456,14 +1337,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         """
         self.cleanup()
 
-        # Start up the API and default registry server
-
-        # We start the registry server first, as the API server config
-        # depends on the registry port - this ordering allows for
-        # retrying the launch on a port clash
-        self.start_with_retry(self.registry_server, 'registry_port', 3,
-                              **kwargs)
-        kwargs['registry_port'] = self.registry_server.bind_port
+        # Start up the API server
 
         self.start_with_retry(self.api_server_multiple_backend,
                               'api_port', 3, **kwargs)
@@ -1585,14 +1459,11 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         is meant to be called during a normal test case sequence.
         """
 
-        # Spin down the API and default registry server
+        # Spin down the API
         self.stop_server(self.api_server_multiple_backend)
-        self.stop_server(self.registry_server)
         if self.include_scrubber:
             self.stop_server(self.scrubber_daemon)
 
-        self._reset_database(self.registry_server.sql_connection)
-
     def run_sql_cmd(self, sql):
         """
         Provides a crude mechanism to run manual SQL commands for backend
@@ -1617,3 +1488,213 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
                 self._attached_server_logs.append(s.log_file)
             self.addDetail(
                 s.server_name, testtools.content.text_content(s.dump_log()))
+
+
+class SynchronousAPIBase(test_utils.BaseTestCase):
+    """A base class that provides synchronous calling into the API.
+
+    This provides a way to directly call into the API WSGI stack
+    without starting a separate server, and with a simple paste
+    pipeline. Configured with multi-store and a real database.
+
+    This differs from the FunctionalTest lineage above in that they
+    start a full copy of the API server as a separate process, whereas
+    this calls directly into the WSGI stack. This test base is
+    appropriate for situations where you need to be able to mock the
+    state of the world (i.e. warp time, or inject errors) but should
+    not be used for happy-path testing where FunctionalTest provides
+    more isolation.
+
+    To use this, inherit and run start_server() before you are ready
+    to make API calls (either in your setUp() or per-test if you need
+    to change config or mocking).
+
+    Once started, use the api_get(), api_put(), api_post(), and
+    api_delete() methods to make calls to the API.
+
+    """
+
+    TENANT = str(uuid.uuid4())
+
+    @mock.patch('oslo_db.sqlalchemy.enginefacade.writer.get_engine')
+    def setup_database(self, mock_get_engine):
+        """Configure and prepare a fresh sqlite database."""
+        db_file = 'sqlite:///%s/test.db' % self.test_dir
+        self.config(connection=db_file, group='database')
+
+        # NOTE(danms): Make sure that we clear the current global
+        # database configuration, provision a temporary database file,
+        # and run migrations with our configuration to define the
+        # schema there.
+        db_api.clear_db_env()
+        engine = db_api.get_engine()
+        mock_get_engine.return_value = engine
+        with mock.patch('logging.config'):
+            # NOTE(danms): The alembic config in the env module will break our
+            # BaseTestCase logging setup. So mock that out to prevent it while
+            # we db_sync.
+            test_utils.db_sync(engine=engine)
+
+    def setup_simple_paste(self):
+        """Setup a very simple no-auth paste pipeline.
+
+        This configures the API to be very direct, including only the
+        middleware absolutely required for consistent API calls.
+        """
+        self.paste_config = os.path.join(self.test_dir, 'glance-api-paste.ini')
+        with open(self.paste_config, 'w') as f:
+            f.write(textwrap.dedent("""
+            [filter:context]
+            paste.filter_factory = glance.api.middleware.context:\
+                ContextMiddleware.factory
+            [filter:fakeauth]
+            paste.filter_factory = glance.tests.utils:\
+                FakeAuthMiddleware.factory
+            [pipeline:glance-api]
+            pipeline = context rootapp
+            [composite:rootapp]
+            paste.composite_factory = glance.api:root_app_factory
+            /v2: apiv2app
+            [app:apiv2app]
+            paste.app_factory = glance.api.v2.router:API.factory
+            """))
+
+    def _store_dir(self, store):
+        return os.path.join(self.test_dir, store)
+
+    def setup_stores(self):
+        """Configures multiple backend stores.
+
+        This configures the API with three file-backed stores (store1,
+        store2, and store3) as well as a os_glance_staging_store for
+        imports.
+
+        """
+        self.config(enabled_backends={'store1': 'file', 'store2': 'file',
+                                      'store3': 'file'})
+        glance_store.register_store_opts(CONF,
+                                         reserved_stores=wsgi.RESERVED_STORES)
+        self.config(default_backend='store1',
+                    group='glance_store')
+        self.config(filesystem_store_datadir=self._store_dir('store1'),
+                    group='store1')
+        self.config(filesystem_store_datadir=self._store_dir('store2'),
+                    group='store2')
+        self.config(filesystem_store_datadir=self._store_dir('store3'),
+                    group='store3')
+        self.config(filesystem_store_datadir=self._store_dir('staging'),
+                    group='os_glance_staging_store')
+
+        glance_store.create_multi_stores(CONF,
+                                         reserved_stores=wsgi.RESERVED_STORES)
+        glance_store.verify_store()
+
+    def setUp(self):
+        super(SynchronousAPIBase, self).setUp()
+
+        self.setup_database()
+        self.setup_simple_paste()
+        self.setup_stores()
+
+    def start_server(self):
+        """Builds and "starts" the API server.
+
+        Note that this doesn't actually "start" anything like
+        FunctionalTest does above, but that terminology is used here
+        to make it seem like the same sort of pattern.
+        """
+        config.set_config_defaults()
+        self.api = config.load_paste_app('glance-api',
+                                         conf_file=self.paste_config)
+
+    def _headers(self, custom_headers=None):
+        base_headers = {
+            'X-Identity-Status': 'Confirmed',
+            'X-Auth-Token': '932c5c84-02ac-4fe5-a9ba-620af0e2bb96',
+            'X-User-Id': 'f9a41d13-0c13-47e9-bee2-ce4e8bfe958e',
+            'X-Tenant-Id': self.TENANT,
+            'Content-Type': 'application/json',
+            'X-Roles': 'admin',
+        }
+        base_headers.update(custom_headers or {})
+        return base_headers
+
+    def api_request(self, method, url, headers=None, data=None,
+                    json=None, body_file=None):
+        """Perform a request against the API.
+
+        NOTE: Most code should use api_get(), api_post(), api_put(),
+              or api_delete() instead!
+
+        :param method: The HTTP method to use (i.e. GET, POST, etc)
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param headers: Optional updates to the default set of headers
+        :param data: Optional bytes data payload to send (overrides @json)
+        :param json: Optional dict structure to be jsonified and sent as
+                     the payload (mutually exclusive with @data)
+        :param body_file: Optional io.IOBase to provide as the input data
+                          stream for the request (overrides @data)
+        :returns: A webob.Response object
+        """
+        headers = self._headers(headers)
+        req = webob.Request.blank(url, method=method,
+                                  headers=headers)
+        if json and not data:
+            data = jsonutils.dumps(json).encode()
+        if data and not body_file:
+            req.body = data
+        elif body_file:
+            req.body_file = body_file
+        return self.api(req)
+
+    def api_get(self, url, headers=None):
+        """Perform a GET request against the API.
+
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param headers: Optional updates to the default set of headers
+        :returns: A webob.Response object
+        """
+        return self.api_request('GET', url, headers=headers)
+
+    def api_post(self, url, headers=None, data=None, json=None,
+                 body_file=None):
+        """Perform a POST request against the API.
+
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param headers: Optional updates to the default set of headers
+        :param data: Optional bytes data payload to send (overrides @json)
+        :param json: Optional dict structure to be jsonified and sent as
+                     the payload (mutually exclusive with @data)
+        :param body_file: Optional io.IOBase to provide as the input data
+                          stream for the request (overrides @data)
+        :returns: A webob.Response object
+        """
+        return self.api_request('POST', url, headers=headers,
+                                data=data, json=json,
+                                body_file=body_file)
+
+    def api_put(self, url, headers=None, data=None, json=None, body_file=None):
+        """Perform a PUT request against the API.
+
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param headers: Optional updates to the default set of headers
+        :param data: Optional bytes data payload to send (overrides @json,
+                     mutually exclusive with body_file)
+        :param json: Optional dict structure to be jsonified and sent as
+                     the payload (mutually exclusive with @data)
+        :param body_file: Optional io.IOBase to provide as the input data
+                          stream for the request (overrides @data)
+        :returns: A webob.Response object
+        """
+        return self.api_request('PUT', url, headers=headers,
+                                data=data, json=json,
+                                body_file=body_file)
+
+    def api_delete(self, url, headers=None):
+        """Perform a DELETE request against the API.
+
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param headers: Optional updates to the default set of headers
+        :returns: A webob.Response object
+        """
+        return self.api_request('DELETE', url, heaers=headers)
