@@ -15,6 +15,7 @@
 
 import atexit
 import os.path
+import shutil
 import tempfile
 
 import fixtures
@@ -117,6 +118,7 @@ CONF = cfg.CONF
 
 
 class ApiTest(test_utils.BaseTestCase):
+
     def setUp(self):
         super(ApiTest, self).setUp()
         self.test_dir = self.useFixture(fixtures.TempDir()).path
@@ -162,8 +164,7 @@ class ApiTest(test_utils.BaseTestCase):
             # use the empty db created and cached as a tempfile
             # instead of spending the time creating a new one
             db_location = os.environ[glance_db_env]
-            test_utils.execute('cp %s %s/tests.sqlite'
-                               % (db_location, self.test_dir))
+            shutil.copyfile(db_location, "%s/tests.sqlite" % self.test_dir)
         else:
             test_utils.db_sync()
 
@@ -171,8 +172,7 @@ class ApiTest(test_utils.BaseTestCase):
             # can be reused for future tests
             (osf, db_location) = tempfile.mkstemp()
             os.close(osf)
-            test_utils.execute('cp %s/tests.sqlite %s'
-                               % (self.test_dir, db_location))
+            shutil.copyfile('%s/tests.sqlite' % self.test_dir, db_location)
             os.environ[glance_db_env] = db_location
 
             # cleanup the temp file when the test suite is
@@ -209,8 +209,8 @@ class ApiTest(test_utils.BaseTestCase):
                 return test_utils.HttplibWsgiAdapter(self.glance_registry_app)
             return wrapped
 
-        self.stubs.Set(glance.common.client.BaseClient,
-                       'get_connection_type', get_connection_type)
+        self.mock_object(glance.common.client.BaseClient,
+                         'get_connection_type', get_connection_type)
 
     def tearDown(self):
         glance.db.sqlalchemy.api.clear_db_env()
