@@ -69,7 +69,7 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
             'glance-direct', 'web-download', 'copy-image'])
         self.config(node_staging_uri='file:///tmp/staging')
         store.create_stores(CONF)
-        self.base_flow = ['ConfigureStaging', 'ImportToStore',
+        self.base_flow = ['ImageLock', 'ConfigureStaging', 'ImportToStore',
                           'DeleteFromFS', 'VerifyImageState',
                           'CompleteTask']
         self.import_plugins = ['Convert_Image',
@@ -78,13 +78,15 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
 
     def _get_flow(self, import_req=None):
         inputs = {
-            'task_id': mock.MagicMock(),
+            'task_id': mock.sentinel.task_id,
             'task_type': mock.MagicMock(),
             'task_repo': mock.MagicMock(),
             'image_repo': mock.MagicMock(),
             'image_id': mock.MagicMock(),
             'import_req': import_req or mock.MagicMock()
         }
+        inputs['image_repo'].get.return_value = mock.MagicMock(
+            extra_properties={'os_glance_import_task': mock.sentinel.task_id})
         flow = api_image_import.get_flow(**inputs)
         return flow
 
@@ -106,8 +108,8 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
         flow = self._get_flow()
 
         flow_comp = self._get_flow_tasks(flow)
-        # assert flow has 5 tasks
-        self.assertEqual(5, len(flow_comp))
+        # assert flow has all the tasks
+        self.assertEqual(len(self.base_flow), len(flow_comp))
         for c in self.base_flow:
             self.assertIn(c, flow_comp)
 
@@ -125,8 +127,8 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
         flow = self._get_flow(import_req=import_req)
 
         flow_comp = self._get_flow_tasks(flow)
-        # assert flow has 6 tasks
-        self.assertEqual(6, len(flow_comp))
+        # assert flow has all the tasks
+        self.assertEqual(len(self.base_flow) + 1, len(flow_comp))
         for c in self.base_flow:
             self.assertIn(c, flow_comp)
         self.assertIn('WebDownload', flow_comp)
@@ -147,8 +149,8 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
         flow = self._get_flow(import_req=import_req)
 
         flow_comp = self._get_flow_tasks(flow)
-        # assert flow has 6 tasks
-        self.assertEqual(6, len(flow_comp))
+        # assert flow has all the tasks
+        self.assertEqual(len(self.base_flow) + 1, len(flow_comp))
         for c in self.base_flow:
             self.assertIn(c, flow_comp)
         self.assertIn('CopyImage', flow_comp)
@@ -164,8 +166,9 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
         flow = self._get_flow()
 
         flow_comp = self._get_flow_tasks(flow)
-        # assert flow has 8 tasks (base_flow + plugins)
-        self.assertEqual(8, len(flow_comp))
+        # assert flow has all the tasks (base_flow + plugins)
+        plugins = CONF.image_import_opts.image_import_plugins
+        self.assertEqual(len(self.base_flow) + len(plugins), len(flow_comp))
         for c in self.base_flow:
             self.assertIn(c, flow_comp)
         for c in self.import_plugins:
@@ -192,8 +195,8 @@ class TestImportTaskFlow(test_utils.BaseTestCase):
         flow = self._get_flow(import_req=import_req)
 
         flow_comp = self._get_flow_tasks(flow)
-        # assert flow has 6 tasks
-        self.assertEqual(6, len(flow_comp))
+        # assert flow has all the tasks (just base and conversion)
+        self.assertEqual(len(self.base_flow) + 1, len(flow_comp))
         for c in self.base_flow:
             self.assertIn(c, flow_comp)
         self.assertIn('CopyImage', flow_comp)
