@@ -14,10 +14,10 @@
 #    under the License.
 
 import os
+from unittest import mock
 
 import glance_store as store
 from glance_store import location
-import mock
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_db import options
@@ -40,8 +40,7 @@ class StoreClearingUnitTest(test_utils.BaseTestCase):
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_MAP', dict())
 
     def _create_stores(self, passing_config=True):
-        """Create known stores. Mock out sheepdog's subprocess dependency
-        on collie.
+        """Create known stores.
 
         :param passing_config: making store driver passes basic configurations.
         :returns: the number of how many store drivers been loaded.
@@ -66,8 +65,7 @@ class MultiStoreClearingUnitTest(test_utils.BaseTestCase):
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_MAP', dict())
 
     def _create_multi_stores(self, passing_config=True):
-        """Create known stores. Mock out sheepdog's subprocess dependency
-        on collie.
+        """Create known stores.
 
         :param passing_config: making store driver passes basic configurations.
         :returns: the number of how many store drivers been loaded.
@@ -81,6 +79,8 @@ class MultiStoreClearingUnitTest(test_utils.BaseTestCase):
 
         self.config(filesystem_store_datadir=self.test_dir,
                     group='fast')
+        self.config(filesystem_store_datadir=self.test_dir2,
+                    group='cheap')
         store.create_multi_stores(CONF)
 
 
@@ -90,7 +90,6 @@ class IsolatedUnitTest(StoreClearingUnitTest):
     Unit test case that establishes a mock environment within
     a testing directory (in isolation)
     """
-    registry = None
 
     def setUp(self):
         super(IsolatedUnitTest, self).setUp()
@@ -106,13 +105,10 @@ class IsolatedUnitTest(StoreClearingUnitTest):
         store.create_stores()
 
         def fake_get_conection_type(client):
-            DEFAULT_REGISTRY_PORT = 9191
             DEFAULT_API_PORT = 9292
 
             if client.port == DEFAULT_API_PORT:
                 return stubs.FakeGlanceConnection
-            elif client.port == DEFAULT_REGISTRY_PORT:
-                return stubs.FakeRegistryConnection(registry=self.registry)
 
         self.patcher = mock.patch(
             'glance.common.client.BaseClient.get_connection_type',
@@ -132,7 +128,6 @@ class MultiIsolatedUnitTest(MultiStoreClearingUnitTest):
     Unit test case that establishes a mock environment within
     a testing directory (in isolation)
     """
-    registry = None
 
     def setUp(self):
         super(MultiIsolatedUnitTest, self).setUp()
@@ -140,9 +135,6 @@ class MultiIsolatedUnitTest(MultiStoreClearingUnitTest):
         lockutils.set_defaults(os.path.join(self.test_dir))
 
         self.config(debug=False)
-        stubs.stub_out_registry_and_store_server(self,
-                                                 self.test_dir,
-                                                 registry=self.registry)
 
     def set_policy_rules(self, rules):
         fap = open(CONF.oslo_policy.policy_file, 'w')
