@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from mox3 import mox
+from unittest import mock
+
 from six.moves import http_client
 import testtools
 
@@ -26,63 +27,40 @@ class TestClient(testtools.TestCase):
 
     def setUp(self):
         super(TestClient, self).setUp()
-        self.mock = mox.Mox()
-        self.mock.StubOutWithMock(http_client.HTTPConnection, 'request')
-        self.mock.StubOutWithMock(http_client.HTTPConnection, 'getresponse')
-
         self.endpoint = 'example.com'
         self.client = client.BaseClient(self.endpoint, port=9191,
                                         auth_token=u'abc123')
 
     def tearDown(self):
         super(TestClient, self).tearDown()
-        self.mock.UnsetStubs()
 
     def test_make_auth_plugin(self):
         creds = {'strategy': 'keystone'}
         insecure = False
-        configure_via_auth = True
 
-        self.mock.StubOutWithMock(auth, 'get_plugin_from_strategy')
-        auth.get_plugin_from_strategy('keystone', creds, insecure,
-                                      configure_via_auth)
-
-        self.mock.ReplayAll()
-
-        self.client.make_auth_plugin(creds, insecure)
-
-        self.mock.VerifyAll()
-
-    def test_http_encoding_headers(self):
-        http_client.HTTPConnection.request(
-            mox.IgnoreArg(),
-            mox.IgnoreArg(),
-            mox.IgnoreArg(),
-            mox.IgnoreArg())
+        with mock.patch.object(auth, 'get_plugin_from_strategy'):
+            self.client.make_auth_plugin(creds, insecure)
 
+    @mock.patch.object(http_client.HTTPConnection, "getresponse")
+    @mock.patch.object(http_client.HTTPConnection, "request")
+    def test_http_encoding_headers(self, _mock_req, _mock_resp):
         # Lets fake the response
         # returned by http_client
         fake = utils.FakeHTTPResponse(data=b"Ok")
-        http_client.HTTPConnection.getresponse().AndReturn(fake)
-        self.mock.ReplayAll()
+        _mock_resp.return_value = fake
 
         headers = {"test": u'ni\xf1o'}
         resp = self.client.do_request('GET', '/v1/images/detail',
                                       headers=headers)
         self.assertEqual(fake, resp)
 
-    def test_http_encoding_params(self):
-        http_client.HTTPConnection.request(
-            mox.IgnoreArg(),
-            mox.IgnoreArg(),
-            mox.IgnoreArg(),
-            mox.IgnoreArg())
-
+    @mock.patch.object(http_client.HTTPConnection, "getresponse")
+    @mock.patch.object(http_client.HTTPConnection, "request")
+    def test_http_encoding_params(self, _mock_req, _mock_resp):
         # Lets fake the response
         # returned by http_client
         fake = utils.FakeHTTPResponse(data=b"Ok")
-        http_client.HTTPConnection.getresponse().AndReturn(fake)
-        self.mock.ReplayAll()
+        _mock_resp.return_value = fake
 
         params = {"test": u'ni\xf1o'}
         resp = self.client.do_request('GET', '/v1/images/detail',
