@@ -14,10 +14,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-import os
+import glance_store as store
 import tempfile
+from unittest import mock
 
+from oslo_config import cfg
 from oslo_log import log as logging
 import six
 import webob
@@ -28,6 +29,9 @@ from glance.common import utils
 from glance.tests import utils as test_utils
 
 
+CONF = cfg.CONF
+
+
 class TestStoreUtils(test_utils.BaseTestCase):
     """Test glance.common.store_utils module"""
 
@@ -359,46 +363,6 @@ class TestUtils(test_utils.BaseTestCase):
         result = utils.mutating(fake_function)
         self.assertEqual("test passed", result(req, Fake()))
 
-    def test_validate_key_cert_key(self):
-        self.config(digest_algorithm='sha256')
-        var_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                               '../../', 'var'))
-        keyfile = os.path.join(var_dir, 'privatekey.key')
-        certfile = os.path.join(var_dir, 'certificate.crt')
-        utils.validate_key_cert(keyfile, certfile)
-
-    def test_validate_key_cert_no_private_key(self):
-        with tempfile.NamedTemporaryFile('w+') as tmpf:
-            self.assertRaises(RuntimeError,
-                              utils.validate_key_cert,
-                              "/not/a/file", tmpf.name)
-
-    def test_validate_key_cert_cert_cant_read(self):
-        with tempfile.NamedTemporaryFile('w+') as keyf:
-            with tempfile.NamedTemporaryFile('w+') as certf:
-                os.chmod(certf.name, 0)
-                self.assertRaises(RuntimeError,
-                                  utils.validate_key_cert,
-                                  keyf.name, certf.name)
-
-    def test_validate_key_cert_key_cant_read(self):
-        with tempfile.NamedTemporaryFile('w+') as keyf:
-            with tempfile.NamedTemporaryFile('w+') as certf:
-                os.chmod(keyf.name, 0)
-                self.assertRaises(RuntimeError,
-                                  utils.validate_key_cert,
-                                  keyf.name, certf.name)
-
-    def test_invalid_digest_algorithm(self):
-        self.config(digest_algorithm='fake_algorithm')
-        var_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                               '../../', 'var'))
-        keyfile = os.path.join(var_dir, 'privatekey.key')
-        certfile = os.path.join(var_dir, 'certificate.crt')
-        self.assertRaises(ValueError,
-                          utils.validate_key_cert,
-                          keyfile, certfile)
-
     def test_valid_hostname(self):
         valid_inputs = ['localhost',
                         'glance04-a'
@@ -482,6 +446,148 @@ class TestUtils(test_utils.BaseTestCase):
                               utils.parse_valid_host_port,
                               pair)
 
+    def test_get_stores_from_request_returns_default(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        req = webob.Request.blank('/some_request')
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, {})
+            self.assertEqual(["ceph1"], result)
+            mock_get_store.assert_called_once_with("ceph1")
+
+    def test_get_stores_from_request_returns_stores_from_body(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        body = {"stores": ["ceph1", "ceph2"]}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, body)
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_returns_store_from_headers(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        headers = {"x-image-meta-store": "ceph2"}
+        req = webob.Request.blank("/some_request", headers=headers)
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, {})
+            self.assertEqual(["ceph2"], result)
+            mock_get_store.assert_called_once_with("ceph2")
+
+    def test_get_stores_from_request_raises_bad_request(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        headers = {"x-image-meta-store": "ceph2"}
+        body = {"stores": ["ceph1"]}
+        req = webob.Request.blank("/some_request", headers=headers)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          utils.get_stores_from_request, req, body)
+
+    def test_get_stores_from_request_returns_all_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        reserved_stores = {
+            'os_glance_staging_store': 'file',
+            'os_glance_tasks_store': 'file'
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF, reserved_stores=reserved_stores)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+            self.assertNotIn('os_glance_staging_store', result)
+            self.assertNotIn('os_glance_tasks_store', result)
+
+    def test_get_stores_from_request_excludes_reserved_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_excludes_readonly_store(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd",
+            "http": "http"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertNotIn("http", result)
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_raises_bad_request_with_all_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        headers = {"x-image-meta-store": "ceph2"}
+        body = {"stores": ["ceph1"], "all_stores": True}
+        req = webob.Request.blank("/some_request", headers=headers)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          utils.get_stores_from_request, req, body)
+
 
 class SplitFilterOpTestCase(test_utils.BaseTestCase):
 
