@@ -15,19 +15,17 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import cfg
 import oslo_utils.importutils
 
-import glance.async
-from glance.async import taskflow_executor
+import glance.async_
+from glance.async_ import taskflow_executor
 from glance.common import exception
-from glance.common.glare import definitions
 from glance.common import timeutils
 from glance import domain
-from glance.glare import domain as artifacts_domain
 import glance.tests.utils as test_utils
 
 
@@ -469,12 +467,12 @@ class TestTask(test_utils.BaseTestCase):
             self.task.expires_at
         )
 
-    @mock.patch.object(glance.async.TaskExecutor, 'begin_processing')
+    @mock.patch.object(glance.async_.TaskExecutor, 'begin_processing')
     def test_run(self, mock_begin_processing):
-        executor = glance.async.TaskExecutor(context=mock.ANY,
-                                             task_repo=mock.ANY,
-                                             image_repo=mock.ANY,
-                                             image_factory=mock.ANY)
+        executor = glance.async_.TaskExecutor(context=mock.ANY,
+                                              task_repo=mock.ANY,
+                                              image_repo=mock.ANY,
+                                              image_factory=mock.ANY)
         self.task.run(executor)
 
         mock_begin_processing.assert_called_once_with(self.task.task_id)
@@ -575,22 +573,3 @@ class TestTaskExecutorFactory(test_utils.BaseTestCase):
         # NOTE(flaper87): "eventlet" executor. short name to avoid > 79.
         te_evnt = task_executor_factory.new_task_executor(context)
         self.assertIsInstance(te_evnt, taskflow_executor.TaskExecutor)
-
-
-class TestArtifact(definitions.ArtifactType):
-    prop1 = definitions.Dict()
-    prop2 = definitions.Integer(min_value=10)
-
-
-class TestArtifactTypeFactory(test_utils.BaseTestCase):
-
-    def setUp(self):
-        super(TestArtifactTypeFactory, self).setUp()
-        context = mock.Mock(owner='me')
-        self.factory = artifacts_domain.ArtifactFactory(context, TestArtifact)
-
-    def test_new_artifact_min_params(self):
-        artifact = self.factory.new_artifact("foo", "1.0.0-alpha")
-        self.assertEqual('creating', artifact.state)
-        self.assertEqual('me', artifact.owner)
-        self.assertIsNotNone(artifact.id)
