@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import copy
+from unittest import mock
 import uuid
 
-import mock
 from mock import patch
 from oslo_utils import encodeutils
 from oslo_utils import units
@@ -43,7 +43,7 @@ class FakeImage(object):
     locations = [{'url': 'file:///not/a/path', 'metadata': {}}]
     tags = set([])
 
-    def set_data(self, data, size=None, backend=None):
+    def set_data(self, data, size=None, backend=None, set_active=True):
         self.size = 0
         for d in data:
             self.size += len(d)
