@@ -13,12 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
 import uuid
 
 import glance_store
-import mock
 from mock import patch
-from mox3 import mox
 from oslo_config import cfg
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
@@ -39,10 +38,8 @@ class TestScrubber(test_utils.BaseTestCase):
         self.config(group='glance_store', default_store='file',
                     filesystem_store_datadir=self.test_dir)
         glance_store.create_stores()
-        self.mox = mox.Mox()
 
     def tearDown(self):
-        self.mox.UnsetStubs()
         # These globals impact state outside of this test class, kill them.
         scrubber._file_queue = None
         scrubber._db_queue = None
@@ -52,13 +49,10 @@ class TestScrubber(test_utils.BaseTestCase):
         uri = 'file://some/path/%s' % uuid.uuid4()
         id = 'helloworldid'
         scrub = scrubber.Scrubber(glance_store)
-        self.mox.StubOutWithMock(glance_store, "delete_from_backend")
-        glance_store.delete_from_backend(
-            uri,
-            mox.IgnoreArg()).AndRaise(ex)
-        self.mox.ReplayAll()
-        scrub._scrub_image(id, [(id, '-', uri)])
-        self.mox.VerifyAll()
+        with patch.object(glance_store,
+                          "delete_from_backend") as _mock_delete:
+            _mock_delete.side_effect = ex
+            scrub._scrub_image(id, [(id, '-', uri)])
 
     @mock.patch.object(db_api, "image_get")
     def test_store_delete_successful(self, mock_image_get):
@@ -66,11 +60,9 @@ class TestScrubber(test_utils.BaseTestCase):
         id = 'helloworldid'
 
         scrub = scrubber.Scrubber(glance_store)
-        self.mox.StubOutWithMock(glance_store, "delete_from_backend")
-        glance_store.delete_from_backend(uri, mox.IgnoreArg()).AndReturn('')
-        self.mox.ReplayAll()
-        scrub._scrub_image(id, [(id, '-', uri)])
-        self.mox.VerifyAll()
+        with patch.object(glance_store,
+                          "delete_from_backend"):
+            scrub._scrub_image(id, [(id, '-', uri)])
 
     @mock.patch.object(db_api, "image_get")
     def test_store_delete_store_exceptions(self, mock_image_get):
@@ -83,13 +75,10 @@ class TestScrubber(test_utils.BaseTestCase):
         ex = glance_store.GlanceStoreException()
 
         scrub = scrubber.Scrubber(glance_store)
-        self.mox.StubOutWithMock(glance_store, "delete_from_backend")
-        glance_store.delete_from_backend(
-            uri,
-            mox.IgnoreArg()).AndRaise(ex)
-        self.mox.ReplayAll()
-        scrub._scrub_image(id, [(id, '-', uri)])
-        self.mox.VerifyAll()
+        with patch.object(glance_store,
+                          "delete_from_backend") as _mock_delete:
+            _mock_delete.side_effect = ex
+            scrub._scrub_image(id, [(id, '-', uri)])
 
     @mock.patch.object(db_api, "image_get")
     def test_store_delete_notfound_exception(self, mock_image_get):
@@ -100,11 +89,10 @@ class TestScrubber(test_utils.BaseTestCase):
         ex = glance_store.NotFound(message='random')
 
         scrub = scrubber.Scrubber(glance_store)
-        self.mox.StubOutWithMock(glance_store, "delete_from_backend")
-        glance_store.delete_from_backend(uri, mox.IgnoreArg()).AndRaise(ex)
-        self.mox.ReplayAll()
-        scrub._scrub_image(id, [(id, '-', uri)])
-        self.mox.VerifyAll()
+        with patch.object(glance_store,
+                          "delete_from_backend") as _mock_delete:
+            _mock_delete.side_effect = ex
+            scrub._scrub_image(id, [(id, '-', uri)])
 
     def test_scrubber_exits(self):
         # Checks for Scrubber exits when it is not able to fetch jobs from
@@ -116,6 +104,21 @@ class TestScrubber(test_utils.BaseTestCase):
         self.assertRaises(exception.FailedToGetScrubberJobs,
                           scrub._get_delete_jobs)
 
+    @mock.patch.object(db_api, "image_restore")
+    def test_scrubber_revert_image_status(self, mock_image_restore):
+        scrub = scrubber.Scrubber(glance_store)
+        scrub.revert_image_status('fake_id')
+
+        mock_image_restore.side_effect = exception.ImageNotFound
+        self.assertRaises(exception.ImageNotFound,
+                          scrub.revert_image_status,
+                          'fake_id')
+
+        mock_image_restore.side_effect = exception.Conflict
+        self.assertRaises(exception.Conflict,
+                          scrub.revert_image_status,
+                          'fake_id')
+
 
 class TestScrubDBQueue(test_utils.BaseTestCase):
 
@@ -175,12 +178,11 @@ class ImagePager(object):
             page_size = image_count
         self.image_batches = []
         start = 0
-        l = len(images)
-        while start < l:
+        while start < image_count:
             self.image_batches.append(images[start: start + page_size])
             start += page_size
-            if (l - start) < page_size:
-                page_size = l - start
+            if (image_count - start) < page_size:
+                page_size = image_count - start
 
     def __call__(self):
         if len(self.image_batches) == 0:
