@@ -16,7 +16,8 @@
 
 from cryptography import exceptions as crypto_exception
 import glance_store as store
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from six.moves import urllib
 
@@ -65,7 +66,7 @@ def sort_url_by_qs_keys(url):
 def get_fake_request(path='', method='POST', is_admin=False, user=USER1,
                      roles=None, tenant=TENANT1):
     if roles is None:
-            roles = ['member']
+        roles = ['member']
 
     req = wsgi.Request.blank(path)
     req.method = method
@@ -230,14 +231,38 @@ class FakeStoreAPI(object):
             size = len(data.data.fd)
         if (current_store_size + size) > store_max_size:
             raise exception.StorageFull()
-        if context.user == USER2:
+        if context.user_id == USER2:
             raise exception.Forbidden()
-        if context.user == USER3:
+        if context.user_id == USER3:
             raise exception.StorageWriteDenied()
         self.data[image_id] = (data, size)
         checksum = 'Z'
         return (image_id, size, checksum, self.store_metadata)
 
+    def add_to_backend_with_multihash(
+            self, conf, image_id, data, size, hashing_algo,
+            scheme=None, context=None, verifier=None):
+        store_max_size = 7
+        current_store_size = 2
+        for location in self.data.keys():
+            if image_id in location:
+                raise exception.Duplicate()
+        if not size:
+            # 'data' is a string wrapped in a LimitingReader|CooperativeReader
+            # pipeline, so peek under the hood of those objects to get at the
+            # string itself.
+            size = len(data.data.fd)
+        if (current_store_size + size) > store_max_size:
+            raise exception.StorageFull()
+        if context.user_id == USER2:
+            raise exception.Forbidden()
+        if context.user_id == USER3:
+            raise exception.StorageWriteDenied()
+        self.data[image_id] = (data, size)
+        checksum = 'Z'
+        multihash = 'ZZ'
+        return (image_id, size, checksum, multihash, self.store_metadata)
+
     def check_location_metadata(self, val, key=''):
         store.check_location_metadata(val)
 
